COMMENT |

	Copyright (C) 1992 Duane Paulson
	NAME: FIXANSI.ASM
	VERSION: 1.01 10/08/92
	PURPOSE: Fixes up ANSI messages that have been thrashed by
		 mail software. Specifically, restores the escape
		 character when it has been altered as in BADESC1 and
		 BADESC2, below.
	SYNTAX: fixansi [dr:][path]filename.ext
	RETURNS: 0 if success, 1 if fail
|

BADESC1		EQU	"`"
BADESC2		EQU	250
TAB		EQU	9
SPACE		EQU	32
BOX		EQU	254
SEEK_SET	EQU	0
SEEK_CUR	EQU	1
SEEK_END	EQU	2

text		SEGMENT	'CODE'
		ORG	80h
count		LABEL	BYTE
		ORG	81h
cmd_tail	LABEL	BYTE
		ORG	100h
		ASSUME	CS:text, DS:text, SS:text, ES:text

main		PROC	NEAR
		CALL	get_filename
		CALL	open_file

; release excess memory - can't be PROC because it changes the stack location.
		mov	ah,4Ah			; reallocate memory (setblock)
		mov	bx,OFFSET lastbyte	; number of bytes in code
		add	bx,2048			; add a 2k stack
		cli				; disable interrupts
		mov	sp,bx			; move stack pointer
		sti				; enable interrupts
		mov	cl,4			; divide by 4 for paragraphs
		shr	bx,cl
		inc	bx			; include last paragraph
		int	21h			; call dos to release memory
		
		CALL	allocate_memory		; allocate a 64K buffer
		mov	dx,OFFSET hello		; opening message
		CALL	show_msg
		CALL	read_file
		CALL	reset_fp
		mov	al,'`'			; load first wonky character
		CALL	fix_wonky		; fix it when found
		mov	al,250			; load other wonky (dot)
		CALL	fix_wonky
		CALL	write_file
		CALL	free_memory		; free the 64K buffer
		mov	al,0			; exit code for success
		CALL	endit
main		ENDP

fix_wonky	PROC	NEAR
		xor	di,di			; clear offset pointer
		mov	cx,filelen		; length of file
next_ag:	repne	scasb			; scan for first matching byte
		jnz	end_fix			; jump if we are
		cmp	BYTE PTR es:[di],'['	; else check for bracket
		jne	next_ag			; loop if not found
		dec	di			; else point to wonky char
		inc	cx
		mov	BYTE PTR es:[di],27	; and change to escape
		jmp	SHORT	next_ag
end_fix:	ret
fix_wonky	ENDP

write_file	PROC	NEAR
		mov	ah,40h			; write file
		mov	bx,handle
		mov	cx,filelen
		push	ds			; swap ds and es
		push	es
		pop	ds
		pop	es
		mov	dx,0			; point to beginning of buffer
		int	21h
		push	ds			; restore ds and es
		push	es
		pop	ds
		pop	es
		ret
write_file	ENDP

read_file	PROC	NEAR
		mov	ah,3Fh			; read file
		mov	bx,handle
		mov	cx,filelen
		push	ds			; swap ds and es
		push	es
		pop	ds
		pop	es
		mov	dx,0			; point to beginning of buffer
		int	21h
		push	ds			; restore ds and es
		push	es
		pop	ds
		pop	es
		ret
read_file	ENDP

open_file	PROC	NEAR
		mov	ax,3D02h		; open file for read/write
		mov	dx,si			; pointer to filename
		int	21h
		jnc	save_handle
		jmp	SHORT	no_file
save_handle:	mov	handle,ax		; save dos handle
		mov	ah,42h			; move file pointer
		mov	al,SEEK_END		; relative to end of file
		mov	bx,handle
		xor	cx,cx			; clear high word of position
		xor	dx,dx			; clear low word
		int	21h
		cmp	dx,0			; dx will be 0 if length <= 64K
		je	reset_file
		jmp	SHORT	too_long
reset_file:	mov	filelen,ax		; save length of file
		CALL	reset_fp
		ret
open_file	ENDP

reset_fp	PROC	NEAR
		mov	ah,42h			; move file pointer
		mov	al,SEEK_SET		; relative to top of file
		mov	bx,handle
		xor	cx,cx			; point to top of file
		xor	dx,dx
		int	21h
		ret
reset_fp	ENDP

get_filename	PROC	NEAR
	;check for no command line arguments
		cmp     count,0
		jne	strip_ws
		jmp	SHORT      help

	; strip white space from front of command line arguments

strip_ws:	mov	bx,OFFSET cmd_tail	; pointer to command args
get_ctail:	cmp	BYTE PTR [bx],SPACE	; is character a space?
		je	next_char		; jump if it is
		cmp	BYTE PTR [bx],TAB	; is character a tab?
		jne	flip_cr			; jump if it isn't
next_char:	inc	bx			; point to next character
		jmp	SHORT	get_ctail	; and loop

	; change Carriage Return terminating cmd line to a null character
flip_cr:	mov	si,bx			; save pointer to filename
		mov	bl,count		; get length of command args
		add	bl,OFFSET cmd_tail	; add pointer to args
		xor	bh,bh			; clear high byte of bx
		mov	BYTE PTR [bx],0		; change CR to null character
		ret
get_filename	ENDP

allocate_memory	PROC	NEAR
		mov	ah,48h			; allocate memory
		mov	bx,4096			; number of paragraphs (64K)
		int	21h
		jc	no_mem
		mov	es,ax			; save buffer segment
		ret
allocate_memory	ENDP

free_memory	PROC	NEAR
		mov	ah,49h			; free allocated memory
		int	21h
		ret
free_memory	ENDP

show_msg	PROC	NEAR			; prints string dx points to
		mov	ah,9			; print string to video
		int	21h
		ret
show_msg	ENDP

error		PROC	NEAR
help:		mov	dx,OFFSET help_msg
		CALL	show_msg
		jmp	SHORT	quit

no_file:	mov	dx,OFFSET no_file1
		CALL	show_msg
		mov	BYTE PTR [bx],'$'	; replace null char with $
		mov	dx,si
		CALL	show_msg		; print filename
		mov	dx,OFFSET no_file2
		CALL	show_msg
		jmp	SHORT	quit

too_long:	mov	dx,OFFSET file2long
		CALL	show_msg
		jmp	SHORT	quit

no_mem:		mov	dx,OFFSET nomemory	; pointer
		CALL	show_msg
		jmp	SHORT	quit

quit:		mov	al,1			; error code for exit
		CALL	endit
error		ENDP

endit		PROC	NEAR
		mov	ah,4Ch			; exit with error code in al
		int	21h
endit		ENDP

;data
handle		DW	?
filelen		DW	?
hello		DB	"Fixansi ",BOX," v1.01 ",BOX
		DB	" Copyright (C) 1992 Duane Paulson",13,10,"$"
help_msg	DB	"fixansi [dr:][path]filename.ext",13,10,"$"
no_file1	DB	'Could not open "$'
no_file2	DB	'" as a file.',13,10,"$"
file2long	DB	"File too long. Must be 65536 bytes or less.",13,10,"$"
nomemory	DB	"Unable to allocate memory for buffer",13,10,"$"
lastbyte	EQU	$
text		ENDS
		END	main
